==> nest-infrastructure-spring-boot-3.5.3.0-20251216.104358-15-sources.jar: com/digiwin/dap/nest/infrastructure/spring/boot/core/config/DwSpringPropertyProcessor.java <==
package com.digiwin.dap.nest.infrastructure.spring.boot.core.config;

import com.digiwin.dap.nest.kernel.core.config.DwConfig;
import com.digiwin.dap.nest.kernel.core.config.DwEnvProperty;
import com.digiwin.dap.nest.kernel.core.config.DwProperty;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.util.datastructure.JaCollectionUtil;
import com.digiwin.dap.nest.kernel.meta.function.JaFunctionP;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringBeanUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringContextUtil;
import org.springframework.boot.context.properties.ConfigurationPropertiesBindingPostProcessor;
import org.springframework.core.env.AbstractEnvironment;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.core.env.MutablePropertySources;
import org.springframework.core.env.PropertiesPropertySource;
import org.springframework.core.env.PropertySource;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;

public class DwSpringPropertyProcessor {

    private static final Properties jaProperties = new Properties();

    private DwSpringPropertyProcessor() {
    }

    // org.springframework.boot.context.config.ConfigFileApplicationListener.Loader.load(org.springframework.boot.env.PropertySourceLoader, java.lang.String, org.springframework.boot.context.config.ConfigFileApplicationListener.Profile, org.springframework.boot.context.config.ConfigFileApplicationListener.DocumentFilter, org.springframework.boot.context.config.ConfigFileApplicationListener.DocumentConsumer)
    private static final String LoadPropertySourcePre = "application";

    /**
     * JaProperty加载到Environment
     */
    public static synchronized void setEnvironment(ConfigurableEnvironment environment, String name) {
        MutablePropertySources propertySources = getMutablePropertySources(environment);
        if (null != propertySources) {
            String relativePropertySourceName = null;
            for (PropertySource<?> propertySource : propertySources) {
                if (null != propertySource && propertySource.getName().contains(LoadPropertySourcePre)) {
                    relativePropertySourceName = propertySource.getName();
                    break;
                }
            }

            DwProperty.getPropertyMap().forEach(jaProperties::put);
            if (null == relativePropertySourceName) {
                propertySources.addLast(new PropertiesPropertySource(name, jaProperties));
            } else {
                propertySources.addBefore(relativePropertySourceName, new PropertiesPropertySource(name, jaProperties));
            }
        }
    }

    /**
     * 最终使用配置输出
     */
    public static void outputFinalConfig(ConfigurableEnvironment environment) {
        if (DwEnvProperty.isCD() || Boolean.TRUE.equals(DwConfig.getBoolean("config.output", false))) {
            try (FileOutputStream outputStream = new FileOutputStream("jugg.properties")) {

                Map<String, Object> map = new TreeMap<>();
                MutablePropertySources propertySources = getMutablePropertySources(environment);
                if (null != propertySources) {
                    ArrayList<Map<String, Object>> allPropertySourcesList = new ArrayList<>();
                    propertySources.forEach(propertySource -> invokeMapPropertySource(propertySource, allPropertySourcesList::add));

                    if (JaCollectionUtil.isNotEmpty(allPropertySourcesList)) {
                        for (int i = allPropertySourcesList.size() - 1; i >= 0; i--) {
                            allPropertySourcesList.get(i).forEach((s, o) -> map.put(s, o.toString()));
                        }
                    }
                }

                Properties properties = new Properties();
                properties.putAll(map);
                properties.store(outputStream, null);
            } catch (Throwable e) {
                DwLog.warn("final.properties store error:{}", e);
            }
        }
    }

    /**
     * 刷新配置
     */
    public static <T> T refresh(Class<T> configBeanClass) {
        try {
            DwProperty.getPropertyMap().forEach(jaProperties::put);
            T targetConfigBean = DwSpringBeanUtil.getBean(configBeanClass);
            ConfigurationPropertiesBindingPostProcessor bindingPostProcessor = DwSpringBeanUtil.getBean(ConfigurationPropertiesBindingPostProcessor.class);
            assert bindingPostProcessor != null;
            assert targetConfigBean != null;
            String beanName = DwSpringContextUtil.getApplicationContext().getBeanNamesForType(configBeanClass)[0];
            bindingPostProcessor.postProcessBeforeInitialization(targetConfigBean, beanName);
            return targetConfigBean;
        } catch (Throwable e) {
            DwLog.error("{} config refresh error:{}", configBeanClass.getName(), e.getMessage());
            return null;
        }
    }

    /**
     * 读取Spring配置到JaProperty
     */
    public static synchronized void loadLocalConfig(ConfigurableEnvironment environment) {
        MutablePropertySources propertySources = getMutablePropertySources(environment);
        if (null != propertySources) {
            ArrayList<Map<String, Object>> applicationConfigPropertySourcesList = new ArrayList<>();
            propertySources.forEach(propertySource -> {
                if (null != propertySource && propertySource.getName().contains(LoadPropertySourcePre)) {
                    invokeMapPropertySource(propertySource, applicationConfigPropertySourcesList::add);
                }
            });

            if (JaCollectionUtil.isNotEmpty(applicationConfigPropertySourcesList)) {
                for (int i = applicationConfigPropertySourcesList.size() - 1; i >= 0; i--) {
                    applicationConfigPropertySourcesList.get(i).forEach((s, o) -> DwProperty.put(s, o.toString()));
                }
            }
        }
    }

    public static MutablePropertySources getMutablePropertySources(ConfigurableEnvironment environment) {
        MutablePropertySources propertySources = null;
        if (environment instanceof AbstractEnvironment) {
            propertySources = environment.getPropertySources();
        } else {
            DwLog.get().warn("not support this environment{}", environment.getClass().getName());
        }
        return propertySources;
    }

    @SuppressWarnings({"unchecked"})
    public static void invokeMapPropertySource(PropertySource<?> propertySource, JaFunctionP<Map<String, Object>> functionP) {
        Object source = propertySource.getSource();
        if (source instanceof Map) {
            Map<String, Object> sourceMap = (Map<String, Object>) source;
            if (JaCollectionUtil.isNotEmpty(sourceMap)) {
                functionP.apply(sourceMap);
            }
        } else {
            DwLog.info("unknown propertySource, name:{}", propertySource.getName());
        }
    }
}

==> nest-infrastructure-spring-boot-5.5.0.1000-20251217.080719-4-sources.jar: com/digiwin/dap/nest/infrastructure/spring/boot/core/config/DwSpringPropertyProcessor.java <==
package com.digiwin.dap.nest.infrastructure.spring.boot.core.config;

import com.digiwin.dap.nest.kernel.core.config.DwConfig;
import com.digiwin.dap.nest.kernel.core.config.DwEnvProperty;
import com.digiwin.dap.nest.kernel.core.config.DwProperty;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.util.datastructure.DwCollectionUtil;
import com.digiwin.dap.nest.kernel.meta.function.JaFunctionP;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringBeanUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringContextUtil;
import org.springframework.boot.context.properties.ConfigurationPropertiesBindingPostProcessor;
import org.springframework.core.env.AbstractEnvironment;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.core.env.MutablePropertySources;
import org.springframework.core.env.PropertiesPropertySource;
import org.springframework.core.env.PropertySource;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;

public class DwSpringPropertyProcessor {

    private static final Properties jaProperties = new Properties();

    private DwSpringPropertyProcessor() {
    }

    // org.springframework.boot.context.config.ConfigFileApplicationListener.Loader.load(org.springframework.boot.env.PropertySourceLoader, java.lang.String, org.springframework.boot.context.config.ConfigFileApplicationListener.Profile, org.springframework.boot.context.config.ConfigFileApplicationListener.DocumentFilter, org.springframework.boot.context.config.ConfigFileApplicationListener.DocumentConsumer)
    private static final String LoadPropertySourcePre = "application";

    /**
     * JaProperty加载到Environment
     */
    public static synchronized void setEnvironment(ConfigurableEnvironment environment, String name) {
        MutablePropertySources propertySources = getMutablePropertySources(environment);
        if (null != propertySources) {
            String relativePropertySourceName = null;
            for (PropertySource<?> propertySource : propertySources) {
                if (null != propertySource && propertySource.getName().contains(LoadPropertySourcePre)) {
                    relativePropertySourceName = propertySource.getName();
                    break;
                }
            }

            DwProperty.getPropertyMap().forEach(jaProperties::put);
            if (null == relativePropertySourceName) {
                propertySources.addLast(new PropertiesPropertySource(name, jaProperties));
            } else {
                propertySources.addBefore(relativePropertySourceName, new PropertiesPropertySource(name, jaProperties));
            }
        }
    }

    /**
     * 最终使用配置输出
     */
    public static void outputFinalConfig(ConfigurableEnvironment environment) {
        if (DwEnvProperty.isCD() || Boolean.TRUE.equals(DwConfig.getBoolean("config.output", false))) {
            try (FileOutputStream outputStream = new FileOutputStream("nest.properties")) {

                Map<String, Object> map = new TreeMap<>();
                MutablePropertySources propertySources = getMutablePropertySources(environment);
                if (null != propertySources) {
                    ArrayList<Map<String, Object>> allPropertySourcesList = new ArrayList<>();
                    propertySources.forEach(propertySource -> invokeMapPropertySource(propertySource, allPropertySourcesList::add));

                    if (DwCollectionUtil.isNotEmpty(allPropertySourcesList)) {
                        for (int i = allPropertySourcesList.size() - 1; i >= 0; i--) {
                            allPropertySourcesList.get(i).forEach((s, o) -> map.put(s, o.toString()));
                        }
                    }
                }

                Properties properties = new Properties();
                properties.putAll(map);
                properties.store(outputStream, null);
            } catch (Throwable e) {
                DwLog.warn("final.properties store error:{}", e);
            }
        }
    }

    /**
     * 刷新配置
     */
    public static <T> T refresh(Class<T> configBeanClass) {
        try {
            DwProperty.getPropertyMap().forEach(jaProperties::put);
            T targetConfigBean = DwSpringBeanUtil.getBean(configBeanClass);
            ConfigurationPropertiesBindingPostProcessor bindingPostProcessor = DwSpringBeanUtil.getBean(ConfigurationPropertiesBindingPostProcessor.class);
            assert bindingPostProcessor != null;
            assert targetConfigBean != null;
            String beanName = DwSpringContextUtil.getApplicationContext().getBeanNamesForType(configBeanClass)[0];
            bindingPostProcessor.postProcessBeforeInitialization(targetConfigBean, beanName);
            return targetConfigBean;
        } catch (Throwable e) {
            DwLog.error("{} config refresh error:{}", configBeanClass.getName(), e.getMessage());
            return null;
        }
    }

    /**
     * 读取Spring配置到JaProperty
     */
    public static synchronized void loadLocalConfig(ConfigurableEnvironment environment) {
        MutablePropertySources propertySources = getMutablePropertySources(environment);
        if (null != propertySources) {
            ArrayList<Map<String, Object>> applicationConfigPropertySourcesList = new ArrayList<>();
            propertySources.forEach(propertySource -> {
                if (null != propertySource && propertySource.getName().contains(LoadPropertySourcePre)) {
                    invokeMapPropertySource(propertySource, applicationConfigPropertySourcesList::add);
                }
            });

            if (DwCollectionUtil.isNotEmpty(applicationConfigPropertySourcesList)) {
                for (int i = applicationConfigPropertySourcesList.size() - 1; i >= 0; i--) {
                    applicationConfigPropertySourcesList.get(i).forEach((s, o) -> DwProperty.put(s, o.toString()));
                }
            }
        }
    }

    public static MutablePropertySources getMutablePropertySources(ConfigurableEnvironment environment) {
        MutablePropertySources propertySources = null;
        if (environment instanceof AbstractEnvironment) {
            propertySources = environment.getPropertySources();
        } else {
            DwLog.get().warn("not support this environment{}", environment.getClass().getName());
        }
        return propertySources;
    }

    @SuppressWarnings({"unchecked"})
    public static void invokeMapPropertySource(PropertySource<?> propertySource, JaFunctionP<Map<String, Object>> functionP) {
        Object source = propertySource.getSource();
        if (source instanceof Map) {
            Map<String, Object> sourceMap = (Map<String, Object>) source;
            if (DwCollectionUtil.isNotEmpty(sourceMap)) {
                functionP.apply(sourceMap);
            }
        } else {
            DwLog.info("unknown propertySource, name:{}", propertySource.getName());
        }
    }
}
